/*
 * Project Name:
     TouchPanel_Controller
 * Copyright:
     (c) MikroElektronika, 2008
 * Revision History:
     20080402:
       - initial release;
 * Description:
     This is a simple demonstration on how to use TouchPanel Controller.
     In this example SPI Glcd library is used to control the Glcd display.
 * Test configuration:
     MCU:             P16F877A
     Dev.Board:       EasyPIC5
     Oscillator:      HS-PLL, 08.0000 MHz
     Ext. Modules:    TouchPanel Controller, Serial Lcd/Glcd adapter, Glcd 128x64-KS108/107
     SW:              mikroC v8.1
 * Notes:
     - Turn off PORTA LEDs, 7Seg Display Switches, ADC jumpers and PORTA pull-up/down resistors.
     - Make sure that the J1 on Lcd/Glcd adapter in in the Graph. position and TouchPanel
         connector is in it's socket.
     - If nothing is shown on Glcd screen, try changing the contrast on Lcd/Glcd adapter
*/

char write_erase, pen_size;
unsigned int x_coord, y_coord;
long         x_coord128, y_coord64;                       // scaled x-y position
int          cal_x_min, cal_y_min, cal_x_max, cal_y_max;  // calibration constants
char write_msg[] = "WRITE";                               // Glcd menu messages
char clear_msg[] = "CLEAR";
char erase_msg[] = "ERASE";

const unsigned int ADC_THRESHOLD = 900;                   // threshold value for press detecting

// returns 1 if TouchPanel is pressed, 0 otherwise
char PressDetect() {
  unsigned adc_rd;
  char result;

  // press detecting
  PORTA.F2 = 0;         // DRIVEA = 0 (LEFT drive off, RIGHT drive off, TOP drive on)
  PORTA.F5 = 0;         // DRIVEB = 0 (BOTTOM drive off)
  Delay_us(3500);

  adc_rd = ADC_read(1);                                   // tead RA1 (READ-Y)

  result = (adc_rd > ADC_THRESHOLD);                      // if logical one is detectet

  // debouncing, repeat detecting after 2ms
  Delay_ms(2);
  adc_rd = ADC_read(1);
  result = result & (adc_rd > ADC_THRESHOLD);
  return result;
}

unsigned int GetX() {
  unsigned int result;
  // reading X
  PORTA.F2 = 1;         // DRIVEA = 1 (LEFT drive on, RIGHT drive on, TOP drive off)
  PORTA.F5 = 0;         // DRIVEB = 0 (BOTTOM drive off)

  Delay_ms(5);
  result = ADC_read(0); // reading X value from RA0 (BOTTOM)
  return result;
}

unsigned int GetY() {
  unsigned int result;
  // reading Y
  PORTA.F2 = 0;         // DRIVEA = 0 (LEFT drive off, RIGHT drive off, TOP drive on)
  PORTA.F5 = 1;         // DRIVEB = 1 (BOTTOM drive on)

  Delay_ms(5);
  result = ADC_read(1); // reading Y value from RA1 (from LEFT)
  return result;
}

void Calibrate() {

  Spi_Glcd_Dot(0,63,1);
  Spi_Glcd_Write_Text("TOUCH BOTTOM LEFT",10,3,1);

  while (!PressDetect());

  // get calibration constants (reading and compensating TouchPanel nonlinearity)
  cal_x_min = GetX() - 10;
  cal_y_min = GetY() - 10;
  Delay_ms(1000);

  Spi_Glcd_Fill(0);
  Spi_Glcd_Dot(127,0,1);
  Spi_Glcd_Write_Text("TOUCH UPPER RIGHT",10,4,1);
  while (!PressDetect()) ;

  // get calibration constants (reading and compensating TouchPanel nonlinearity)
  cal_x_max = GetX() + 5;
  cal_y_max = GetY() + 5;
  Delay_ms(1000);
}


void main() {
  ADCON1 = 0x07;         // configure RA0 and RA1 as analog pins,
                         // RA2 and RA5 as digital pins
  PORTA  = 0x00;         // configure RA0 and RA1 as inputs,
  TRISA  = 0x03;         //   RA2 and RA5 as ouputs

  PORTC  = 0 ;           // set PORTC to 0
  TRISC  = 0 ;           // PORTC is output

  Spi_Init();                              // initalize SPI module
  Spi_Glcd_Init(0, &PORTC, 0, &PORTC, 1);  // Spi_Glcd_Init, see Autocomplete

  Spi_Glcd_Fill(0);
  Spi_Glcd_Set_Font(FontSystem5x8, 5, 8, 32);
  Spi_Glcd_Write_Text("CALIBRATION", 24, 3, 1);
  Delay_ms(1500);

  Spi_Glcd_Fill(0);
  Calibrate();
  Spi_Glcd_Fill(0);

  Spi_Glcd_Write_Text("WRITE ON SCREEN", 24, 4, 1) ;
  Delay_ms(1000);
  Spi_Glcd_Fill(0);

  Spi_Glcd_Fill(0);
  Spi_Glcd_V_Line(0,7,0,1);
  Spi_Glcd_Write_Text(clear_msg,1,0,0);
  Spi_Glcd_V_Line(0,7,97,1);
  Spi_Glcd_Write_Text(erase_msg,98,0,0);

  // pen menu:
  Spi_Glcd_Rectangle(41,0,52,9,1);
  Spi_Glcd_Box(45,3,48,6,1);
  Spi_Glcd_Rectangle(63,0,70,7,1);
  Spi_Glcd_Box(66,3,67,4,1);
  Spi_Glcd_Rectangle(80,0,86,6,1);
  Spi_Glcd_Dot(83,3,1);

  write_erase = 1;
  pen_size = 1;

  while (1) {

    if (PressDetect()) {
      // after a press is detected read X-Y and convert it to 128x64 space
      x_coord = GetX() - cal_x_min;
      y_coord = GetY() - cal_y_min;

      x_coord128 = (x_coord * 128l) / (cal_x_max - cal_x_min);
      y_coord64 = (64 -(y_coord *64) / (cal_y_max - cal_y_min));

      if ((x_coord128 < 0) || (x_coord128 > 127))
         continue;
      if ((y_coord64 < 0) || (y_coord64 > 63))
         continue;


      // if clear is pressed
      if ((x_coord128 < 31) && (y_coord64 < 8)) {

         Spi_Glcd_Fill(0);

         // pen menu:
         Spi_Glcd_Rectangle(41,0,52,9,1);
         Spi_Glcd_Box(45,3,48,6,1);
         Spi_Glcd_Rectangle(63,0,70,7,1);
         Spi_Glcd_Box(66,3,67,4,1);
         Spi_Glcd_Rectangle(80,0,86,6,1);
         Spi_Glcd_Dot(83,3,1);

         Spi_Glcd_V_Line(0,7,0,1);
         Spi_Glcd_Write_Text(clear_msg,1,0,0);
         Spi_Glcd_V_Line(0,7,97,1);
         if (write_erase)
           Spi_Glcd_Write_Text(erase_msg,98,0,0);
         else
           Spi_Glcd_Write_Text(write_msg,98,0,0);
         }

      // if write/erase is pressed
      if ((x_coord128 > 96) && (y_coord64 < 8)) {
         if (write_erase) {
           write_erase = 0;
           Spi_Glcd_Write_Text(write_msg,98,0,0);
           Delay_ms(500);
           }
         else {
           write_erase = 1;
           Spi_Glcd_Write_Text(erase_msg,98,0,0);
           Delay_ms(500);
           }
         }

      // if pen size is selected
      if ((x_coord128 >= 41) && (x_coord128 <= 52) && (y_coord64 <= 9))
         pen_size = 3;

      if ((x_coord128 >= 63) && (x_coord128 <= 70) && (y_coord64 <= 7))
         pen_size = 2;

      if ((x_coord128 >= 80) && (x_coord128 <= 86) && (y_coord64 <= 6))
         pen_size = 1;

      if (y_coord64 < 11)
         continue;

      switch (pen_size) {
         case 1 : Spi_Glcd_Dot(x_coord128, y_coord64, write_erase); break;
         case 2 : Spi_Glcd_Box(x_coord128, y_coord64, x_coord128 + 1, y_coord64 + 1, write_erase); break;
         case 3 : Spi_Glcd_Box(x_coord128-1, y_coord64-1, x_coord128 + 2, y_coord64 + 2, write_erase); break;
         }

      }
    }
}